package com.ruoyi.framework.recovery.mapper;

import java.util.List;
import com.ruoyi.framework.recovery.domain.RecContent;
import com.ruoyi.framework.recovery.domain.RecContentUser;
import com.ruoyi.framework.recovery.domain.dto.ContentDto;
import com.ruoyi.framework.recovery.domain.vo.WxContent;
import org.apache.ibatis.annotations.Param;

/**
 * 评论Mapper接口
 *
 * @author jin
 * @date 2024-11-16
 */
public interface RecContentMapper
{
    /**
     * 查询评论
     *
     * @param id 评论主键
     * @return 评论
     */
    public RecContent selectRecContentById(Long id);

    /**
     * 查询评论列表
     *
     * @param recContent 评论
     * @return 评论集合
     */
    public List<RecContent> selectRecContentList(RecContent recContent);

    /**
     * 新增评论
     *
     * @param recContent 评论
     * @return 结果
     */
    public int insertRecContent(RecContent recContent);

    /**
     * 修改评论
     *
     * @param recContent 评论
     * @return 结果
     */
    public int updateRecContent(RecContent recContent);

    /**
     * 删除评论
     *
     * @param id 评论主键
     * @return 结果
     */
    public int deleteRecContentById(Long id);

    /**
     * 批量删除评论
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteRecContentByIds(Long[] ids);

    /**
     * 插入微信用户点赞
     *
     * @param dto 入参
     * @return 结果集
     */
    int insertRecContentUser(@Param("dto") ContentDto dto);

    /**
     * 根据评论id和用户编号查询当前用户是否评论过
     *
     * @param wxOpenid 用户id
     * @param id 评论id
     * @return 结果集
     */
    List<RecContentUser> selectContentUserByOpenId(@Param("openId") String wxOpenid, @Param("contentId") Long id);

    /**
     * 查询评论
     *
     * @param wxContentVo 入参
     * @return 结果集
     */
    List<WxContent> selectWxContentList(@Param("dto") WxContent wxContentVo);

    /**
     * 查询当前登录人是否点过赞
     *
     * @param wxOpenid 登录人编号
     * @return 结果集
     */
    List<WxContent> selectRecContentByOpenId(@Param("openId") String wxOpenid);
}
